import os
from pathlib import Path

from kontur.clusterconfig.default_config import DEFAULT_LOCAL_DIRECTORY, DEFAULT_CLUSTERCONFIG_FILE
from kontur.clusterconfig.settings import Settings


def find_directory_up_the_tree(dirname: str, config_filename: str = DEFAULT_CLUSTERCONFIG_FILE) -> str:
    """
    Поиск директории, вверх по иерархии директорий.

    :param str dirname: Название директории, которую мы ищем.
    :param str config_filename: Имя файла с базовой конфигурацией

    :return: Абсолютный Путь до директории.
    :rtype: str

    """
    current_directory = Path.cwd()

    while True:
        dir_to_check = current_directory / dirname
        cluster_config_file = dir_to_check / config_filename
        if dir_to_check.exists() and cluster_config_file.exists():
            return str(dir_to_check)

        parent_dir = current_directory.parent
        if parent_dir == current_directory:
            break

        current_directory = parent_dir

    return str(Path.cwd() / dirname)


def load_cc_settings(
        settings_dir_name: str = DEFAULT_LOCAL_DIRECTORY,
        config_filename: str = DEFAULT_CLUSTERCONFIG_FILE
) -> Settings:
    """
    Ищем на диске файлы с Clusterconfig и инициализирует объект Settings с этими файлами.

    :param str settings_dir_name: Название папки, в которой будем искать.
    :param str config_filename: Название файла, который содержит clusterconfig

    :return: Проинициализированный объект Settings
    :rtype: Settings

    """
    settings_dir = find_directory_up_the_tree(settings_dir_name)
    cc_config_filename = os.path.join(settings_dir, config_filename)
    return Settings(cc_config_filename)


class Empty:
    """
    Класс, символизирующий None.
    Используется, что бы при поиске ключа настройки,
    можно было различить ситуации когда ключ етсь,
    а значение None и когда ключа нет.

    """
