import os
from typing import List


class FileReaderMixin(object):

    def _get_setting_files(self, path: str) -> List[str]:
        """
        Собираем все пути до всех файлов с конфигами.

        :param str path: Путь до папки, в которой будем собирать все файлы с конфигами.

        :return: Список путей до файлов с конфигами.
        :rtype: List[str]

        """
        files = []
        for (dirpath, dirnames, filenames) in os.walk(path):
            files.extend([os.path.join(dirpath, file) for file in filenames])
        return files

    def _get_last_modified_ts(self, path: str, files: List[str]) -> float:
        """
        Возвращает самое свежее время изменения, среди всех файлов в иерархии файлов конфига.

        :param str path: Путь до корня с файлами конфигов.
        :param List[str] files: Список путей до каждого файла с конфигом.

        :return: Самое позднее время изменения файла.
        :rtype: float

        """
        times = [os.path.getmtime(path)]
        times.extend([os.path.getmtime(file) for file in files])
        return max(times)
