from typing import Tuple, Iterable


class LinesParserMixin(object):
    separators: Iterable[str] = (':=', '=')

    def _processing_line(self, line: str, save_dict: dict, prefix: str = '') -> dict:
        """
        Обработка строки из файла и созранение ее в save_list.

        :param str line: СТрока для обработки.
        :param dict save_dict: Словарь, в который стоит сохранить результат.
        :param str prefix: Префикс, который устанавливается перед key при сохранении распарсенного значения.

        :return: Dict со старыми конфигами и тем, что только что распарсили.
        :rtype: dict

        """
        line = line.strip()
        if line == '':
            return save_dict

        k, v = self._parse_settings_file_line(line)
        if prefix:
            k = prefix + '/' + k
        k = k.strip('/')

        if not (k in save_dict):
            save_dict[k] = v
        elif not isinstance(save_dict[k], list):
            save_dict[k] = [save_dict[k], v]
        else:
            save_dict[k].append(v)

        return save_dict

    def _parse_settings_file_line(self, line: str) -> Tuple[str, str]:
        """
        Парсинг одной строки.

        :param str line: Строка, которую нужно распарсить.

        :return: Распарсенные значения. Tuple[key, value]
        :rtype: Tuple[str, str]

        """
        for separator in self.separators:
            sep_idx = line.find(separator)
            if sep_idx != -1:
                key = line[0:sep_idx].strip()
                value = line[(sep_idx + len(separator)):].strip()
                return key.lower(), value
        return '', line.strip()
