import asyncio
import struct

from hercules_sender.config import Config


async def tcp_reader(reader: asyncio.StreamReader,
                     writer: asyncio.StreamWriter):
    config = Config(None)
    logger = config.logger
    timeout = 30
    while not reader.at_eof():
        try:
            logger.debug("TCPReader. Start message. Format lels")
            data_event_size = await asyncio.wait_for(reader.readexactly(4),
                                                     timeout=timeout)
            event_size = struct.unpack(">i", data_event_size)[0]
            logger.debug("TCPReader. Event size: {}".format(event_size))
            event = await asyncio.wait_for(reader.readexactly(event_size),
                                           timeout=timeout)
            logger.debug("TCPReader. Received event size: {}".format(len(event)))
            data_stream_name_size = await asyncio.wait_for(
                reader.readexactly(1), timeout=timeout)
            stream_name_size = struct.unpack("B", data_stream_name_size)[0]
            logger.debug("TCPReader. Stream name size: {}".format(stream_name_size))

            if stream_name_size == 0:
                logger.debug("TCPReader. Stream name is zero. End message")
                continue

            data_stream_name = await asyncio.wait_for(
                reader.readexactly(stream_name_size), timeout=timeout)
            stream_name = data_stream_name.decode()
            logger.debug("TCPReader. Stream name: {}".format(stream_name))
            logger.debug("TCPReader. End message")
        except struct.error:
            logger.warning(
                "TCPReader. Message is uncomplete. struct.error. Connection closed")
            writer.close()
            return
        except asyncio.IncompleteReadError:
            logger.warning(
                "TCPReader. Message is uncomplete. IncompleteReadError. Connection closed"
            )
            writer.close()
            return
        except asyncio.TimeoutError:
            logger.warning(
                "TCPReader. Message is uncomplete. TimeoutError. Connection closed")
            writer.close()
            return
        logger.debug("TCPReader. Put event in Queue")
        await config.queue.put((stream_name, event))
    writer.write_eof()
