import asyncio
import struct

from hercules_sender.config import Config


async def tcp_reader(reader: asyncio.StreamReader,
                     writer: asyncio.StreamWriter):
    config = Config(None)
    logger = config.logger
    timeout = 30
    max_buffer_size = config.reader_buffer
    buffer = b""
    while not reader.at_eof():
        try:
            logger.debug("TCPReader. Start message. Format lles")
            while len(buffer) < 5:
                local_buffer = await asyncio.wait_for(reader.read(max_buffer_size), timeout=timeout)
                buffer += local_buffer
            event_size, stream_name_size = struct.unpack(">iB", buffer[0:5])
            logger.debug("TCPReader. Event size: {}, Stream name size: {}".format(event_size, stream_name_size))

            while len(buffer) < 5 + event_size + stream_name_size:
                local_buffer = await asyncio.wait_for(reader.read(max_buffer_size), timeout=timeout)
                buffer += local_buffer

            if stream_name_size == 0:
                logger.debug("TCPReader. Stream name is zero. End message")
                buffer = buffer[5 + event_size + stream_name_size:]
                continue

            data_event = buffer[5:5 + event_size]
            data_stream_name = buffer[5 + event_size:5 + event_size + stream_name_size]
            buffer = buffer[5 + event_size + stream_name_size:]

            stream_name = data_stream_name.decode()
            logger.debug("TCPReader. Stream name: {}".format(stream_name))
            logger.debug("TCPReader. End message")
        except struct.error:
            logger.warning(
                "TCPReader. Message is uncomplete. struct.error. Connection closed")
            writer.close()
            return
        except asyncio.IncompleteReadError:
            logger.warning(
                "TCPReader. Message is uncomplete. IncompleteReadError. Connection closed"
            )
            writer.close()
            return
        except asyncio.TimeoutError:
            logger.warning(
                "TCPReader. Message is uncomplete. TimeoutError. Connection closed")
            writer.close()
            return
        except asyncio.CancelledError:
            logger.info(
                "TCPReader. Async task is cancelled. Connection closed")
            writer.close()
        logger.debug("TCPReader. Put event in Queue")
        await config.queue.put((stream_name, data_event))
    writer.write_eof()
